const Clutter		= imports.gi.Clutter;
const GObject		= imports.gi.GObject;
const St			= imports.gi.St;
const PanelMenu		= imports.ui.panelMenu;
const Me			= imports.misc.extensionUtils.getCurrentExtension();
const AppManager	= Me.imports.AppManager;
const getSettings	= Me.imports.utils.getSettings;
const GLib			= imports.gi.GLib;

var TrayIndicator = new imports.lang.Class({
	Name: 'TrayIndicator',
	Extends: PanelMenu.Button,

	_init() {
		this._icons = [];

		this.parent(0.0, null, true);

		this._indicators = new St.BoxLayout();
		this._indicators.set_style('margin-left: 6px');
		this.add_child(this._indicators);

		this.style_class = 'nope';
	},

	get size() {
		const context = St.ThemeContext.get_for_stage(global.stage);
		return getSettings().get_int('icon-size') * context.scale_factor;
	},

	addIcon(icon) {
		icon.opacity = 0;

		icon.connect('queue-relayout', () => {
			GLib.timeout_add(GLib.PRIORITY_DEFAULT, 100, () => {
				icon.set_size(this.size, this.size);
				icon.set_y_align(Clutter.ActorAlign.CENTER);
				icon.opacity = 255;
				return GLib.SOURCE_REMOVE;
			});
		});
		icon.connect('destroy', () => { ibtn.destroy(); });

		const ibtn = new St.Button({ child: icon, button_mask: St.ButtonMask.ONE | St.ButtonMask.TWO | St.ButtonMask.THREE, style: 'padding: 0 8px; margin: 0 2px' });
		this._indicators.add_child(ibtn);

		ibtn.connect('button-release-event', (actor, event) => { 
			if(event) {
				switch(event.get_button()) {
					case 1: AppManager.toggleWindows(icon, event);		break;
					case 2: AppManager.killWindows(icon, event);		break;
					case 3: icon.click(event);							break;
				}
			}
		});

		this._icons.push(icon);
	},

	removeIcon(icon) {
		const actor = icon.get_parent() || icon;
		actor.destroy();

		const index = this._icons.indexOf(icon);
		this._icons.splice(index, 1);
	},

	resizeIcons() {
		this._icons.forEach(icon => {
			icon.set_size(this.size, this.size);
		});
	}
});