const Main				= imports.ui.main;
const Shell				= imports.gi.Shell;
const Me				= imports.misc.extensionUtils.getCurrentExtension();
const TrayIndicator		= Me.imports.TrayIndicator;
const getSettings		= Me.imports.utils.getSettings;
const System			= imports.system;

var TrayIconsClass = new imports.lang.Class({
	Name: 'Tray Icons: Reloaded',

	_init() {
		_settings = getSettings();

		this.tray       = new Shell.TrayManager();
		this.indicators = new TrayIndicator.TrayIndicator();

		this.tray.connect('tray-icon-added', this._onIconAdded.bind(this));
		this.tray.connect('tray-icon-removed', this._onIconRemoved.bind(this));

		Main.panel.addToStatusArea('TrayIconsReloaded', this.indicators, 0, _settings.get_string('tray-position'));

		this.tray.manage_screen(Main.panel);

		this._onChange();
	},

	_onIconAdded(trayManager, icon)		 {		this.indicators.addIcon(icon); 		},
	_onIconRemoved(trayManager, icon)	 {		this.indicators.removeIcon(icon);	},

	_onChange() {
		_settings.connect('changed::icon-size', () => {
			this.indicators.resizeIcons();
		});

        _settings.connect('changed::tray-position', () => {
			Main.panel.addToStatusArea('TrayIconsReloaded' + Math.random(), this.indicators, 0, _settings.get_string('tray-position'));
		});
	},

	_destroy() {
		this.tray = null;

		this.indicators.destroy();
		System.gc();
	}
});

let TrayIcons;

class Extension {
    enable() {
        TrayIcons = new TrayIconsClass();
    }

    disable() { 
		TrayIcons._destroy();
	}
}

function init() {
    return new Extension();
}